@@ -2,6 +2,7 @@
 #include <string>
 #include <stdexcept>
 #include <xs/xs.h>
+#include <stdio.h>
 
 namespace xs {
 
@@ -86,6 +87,54 @@ SV* call_next (pTHX_ CV* cv, SV** args, I32 items, next_t type, I32 flags) {
     return ret;
 }
 
+I32 _call_sub (pTHX_ CV* cv, I32 flags, SV** ret, I32 maxret, AV** aref, SV* first_arg, SV** rest_args, I32 rest_items) {
+    dSP; ENTER; SAVETMPS;
+    PUSHMARK(SP);
+    if (first_arg) XPUSHs(first_arg);
+    for (I32 i = 0; i < rest_items; ++i) XPUSHs(*rest_args++);
+    PUTBACK;
+    if (maxret <= 0 && !aref) { flags |= G_DISCARD; maxret = 0; }
+    I32 count = call_sv((SV*)cv, flags);
+    I32 nret = count > maxret ? maxret : count;
+    SPAGAIN;
+
+    //printf("sub=%s count=%d maxret=%d aref=%d\n", GvNAME(CvGV(cv)), count, maxret, aref ? 1 : 0);
+    if (!aref) {
+        while (count > maxret) { POPs; --count; }
+        while (count > 0) ret[--count] = SvREFCNT_inc_NN(POPs);
+    }
+    else if (count) {
+        AV* arr = *aref = newAV();
+        av_extend(arr, count-1);
+        AvFILLp(arr) = count-1;
+        SV** svlist = AvARRAY(arr);
+        while (count--) svlist[count] = SvREFCNT_inc_NN(POPs);
+    }
+    else *aref = NULL;
+
+    PUTBACK;
+    FREETMPS; LEAVE;
+
+    if (aref && *aref) sv_2mortal((SV*)*aref);
+    else for (I32 i = 0; i < nret; ++i) sv_2mortal(ret[i]);
+
+    return nret;
+}
+
+I32 _call_method (pTHX_ SV* obj, I32 flags, const char* name, STRLEN len, SV** ret, I32 maxret, AV** aref, SV** args, I32 items) {
+    HV* stash = NULL;
+    if (SvROK(obj)) {
+        SV* sv = SvRV(obj);
+        if (SvOBJECT(sv)) stash = SvSTASH(sv);
+    }
+    if (!stash) stash = gv_stashsv(obj, GV_ADD);
+
+    GV* methgv = gv_fetchmethod_pvn_flags(stash, name, len, GV_CROAK);
+
+    return _call_sub(aTHX_ GvCV(methgv), flags, ret, maxret, aref, obj, args, items);
+}
+
+
 /* should be called when interpreter is cloned. If we get here then our perl_object is cloned, that is it is present in PL_ptr_table
  * So we only need to get the new pointer instead of the old one */
 void XSBackref::on_perl_dup (pTHX_ int32_t refcnt) {
