@@ -16,9 +16,9 @@ extern "C" {
 #  define CPP11X
 #endif
 
+#include <algorithm_perlsafe> // safe c++11 compilation
 #include <panda/def.h>
 #include <panda/cast.h>
-#include <panda/traits.h>
 #include <panda/refcnt.h>
 
 #include <pthread.h>
@@ -169,6 +169,50 @@ inline SV* call_super       (pTHX_ CV* cv, SV** args, I32 items, I32 flags = 0)
 inline SV* call_next_method (pTHX_ CV* cv, SV** args, I32 items, I32 flags = 0) { return call_next(aTHX_ cv, args, items, NEXT_METHOD, flags); }
 inline SV* call_next_maybe  (pTHX_ CV* cv, SV** args, I32 items, I32 flags = 0) { return call_next(aTHX_ cv, args, items, NEXT_MAYBE, flags); }
 
+I32 _call_sub    (pTHX_ CV* cv, I32 flags, SV** ret, I32 maxret, AV** aref, SV* first_arg, SV** rest_args, I32 rest_items);
+I32 _call_method (pTHX_ SV* obj, I32 flags, const char* name, STRLEN len, SV** ret, I32 maxret, AV** aref, SV** args, I32 items);
+
+inline void call_sub_void (pTHX_ CV* cv, SV** args = NULL, I32 items = 0) {
+    _call_sub(aTHX_ cv, G_VOID, NULL, 0, NULL, NULL, args, items);
+}
+
+inline SV* call_sub_scalar (pTHX_ CV* cv, SV** args = NULL, I32 items = 0, I32 flags = 0) {
+    SV* ret = NULL;
+    _call_sub(aTHX_ cv, flags|G_SCALAR, &ret, 1, NULL, NULL, args, items);
+    return ret;
+}
+
+inline I32 call_sub_list (pTHX_ CV* cv, SV** ret, I32 maxret, SV** args = NULL, I32 items = 0, I32 flags = 0) {
+    return _call_sub(aTHX_ cv, flags|G_ARRAY, ret, maxret, NULL, NULL, args, items);
+}
+
+inline AV* call_sub_av (pTHX_ CV* cv, SV** args = NULL, I32 items = 0, I32 flags = 0) {
+    AV* ret;
+    _call_sub(aTHX_ cv, flags|G_ARRAY, NULL, 0, &ret, NULL, args, items);
+    return ret;
+
+}
+
+inline void call_method_void (pTHX_ SV* obj, const char* name, STRLEN len, SV** args = NULL, I32 items = 0) {
+    _call_method(aTHX_ obj, G_VOID, name, len, NULL, 0, NULL, args, items);
+}
+
+inline SV* call_method_scalar (pTHX_ SV* obj, const char* name, STRLEN len, SV** args = NULL, I32 items = 0, I32 flags = 0) {
+    SV* ret = NULL;
+    _call_method(aTHX_ obj, flags|G_SCALAR, name, len, &ret, 1, NULL, args, items);
+    return ret;
+}
+
+inline I32 call_method_list (pTHX_ SV* obj, const char* name, STRLEN len, SV** ret, I32 maxret, SV** args = NULL, I32 items = 0, I32 flags = 0) {
+    return _call_method(aTHX_ obj, flags|G_ARRAY, name, len, ret, maxret, NULL, args, items);
+}
+
+inline AV* call_method_av (pTHX_ SV* obj, const char* name, STRLEN len, SV** args = NULL, I32 items = 0, I32 flags = 0) {
+    AV* ret;
+    _call_method(aTHX_ obj, flags|G_ARRAY, name, len, NULL, 0, &ret, args, items);
+    return ret;
+}
+
 bool register_package (pTHX_ const char* module, const char* source_module);
 void inherit_package  (pTHX_ const char* module, const char* parent);
 
